@@ -55,7 +55,7 @@
  *
  * Wed Jul 21 1999 Yuan-Chung Cheng <platin@linux.org.tw>
  * - use properties.os2->ulCodePageRange1 to lookup
- *   font encoding to support modren Unicode encoded 
+ *   font encoding to support modern Unicode encoded 
  *   CJK fonts (platform=3, encoding=1). 
  *   Has been tested with Arphic's and DynaLab's CJK ttf fonts.
  * - foundry ARPH added
@@ -76,7 +76,11 @@
 #include <ctype.h>
 #include <errno.h>
 
-#include <freetype/freetype.h>
+#include <ft2build.h>
+#include FT_FREETYPE_H
+#include FT_SFNT_NAMES_H
+#include FT_TRUETYPE_IDS_H
+#include FT_TRUETYPE_TABLES_H
 
 #include "panose.h"
 #include "ttos2val.h"
@@ -110,21 +114,25 @@ enum completeness {
 
 static int   parse_argv        (int argc, char *argv[]);
 static void  usage             (const char *name);
-static const char *get_foundry (const TT_Char orig_code[4]);
-static const char *get_name    (TT_Face face);
+static const char *get_foundry (const FT_Char orig_code[4]);
+static const char *get_name    (FT_Face face);
+static const char *get_psname  (FT_Face face);
+static const char *get_psname  (FT_Face face);
 static void  add_mapping (struct xlfdmapping **listhead,
 			  char *file_name,
 			  const char *foundry_name,
 			  const char *font_name,
 			  const char *weight_name,
 			  const char *width_name,
-			  TT_Face_Properties *properties,
+			  TT_OS2 *os2,
+			  TT_Postscript *post,
 			  char *encodingscheme
 			  );
 
-static int   has_8bit_encoding (TT_CharMap *cmap, const int *table);
-static const char *get_weight (const char *name, TT_Face_Properties *properties);
-static const char *get_width_name (const char *name, TT_Face_Properties *properties);
+static void  free_mappings (struct xlfdmapping **listhead);
+static int   has_8bit_encoding (FT_Face face, const int *table);
+static const char *get_weight (const char *name, TT_OS2 *os2);
+static const char *get_width_name (const char *name, TT_OS2 *os2);
 
 
 static enum completeness completeness = STRICT;
@@ -677,19 +685,24 @@ main (int argc, char *argv[])
 {
     int n = 0;
     int i;
-    TT_Error fterror;
-    TT_Engine engine;
+    FT_Error fterror;
+    FT_Library library;
     struct xlfdmapping  *head = NULL;
     struct xlfdmapping  *p;
-    TT_Face face;
-    TT_Face_Properties  properties;
+    FT_Face face;
+    TT_OS2 *os2;
+    TT_Postscript *post;
     const char *font_name;
+    const char *font_psname;
     const char *foundry_name;
     const char *weight_name;
     const char *width_name;
     char *platStr=NULL;
     char *encoStr=NULL;
     char  tempStr[128];
+    FT_ULong face_num;
+    FT_ULong face_index = 0;
+    char ch;
 
     /* first parse the command line */
     if (parse_argv (argc, argv) < 0) {
@@ -697,49 +710,78 @@ main (int argc, char *argv[])
 	return TTMKFS_USAGE;
     }
     /* initialize the FreeType library */
-    fterror = TT_Init_FreeType (&engine);
+    fterror = FT_Init_FreeType (&library);
 
-    if (fterror != TT_Err_Ok) {
-	fprintf (stderr, "TT_Init_FreeType failed with %ld\n", fterror);
+    if (fterror) {
+	fprintf (stderr, "FT_Init_FreeType failed with %d\n", fterror);
 	return TTMKFS_FTINIT;
     }
     
     
     /* open the TTF file */
-    fterror = TT_Open_Face (engine, path, &face);
+    fterror = FT_New_Face (library, path, 0, &face);
 	
-    if (fterror != TT_Err_Ok) {
-      fprintf (stderr, "TT_Open_Face failed with %ld for file %s\n", fterror, path);
+    if (fterror != FT_Err_Ok) {
+      fprintf (stderr, "FT_Open_Face failed with %d for file %s\n", fterror, path);
       return TTMKFS_USAGE;
     }
 
-    /* ask FreeType for the font properties */
-    fterror = TT_Get_Face_Properties (face, &properties);
+    face_num = face->num_faces;
+    /* ask user to specify a face if there are more then one face in the font */    if(face_num > 1){
+        /* print the name of every font faces */
+        fprintf (stderr, "%lu font faces found in %s\n", face_num, path);
+        
+        for(face_index = 0; face_index < face_num; face_index++){
+            FT_New_Face(library, path, face_index, &face);
+            fprintf (stderr, "  Face #%lu: %s\n", face_index, get_name (face));
+        }
+        /* ask user to select one */
+        /* if the user enters garbage, then use the first face as default */
+        face_index = 0;
+        fprintf(stderr, "Select a font face (default: 0): ");
+        while((ch = getchar()) && ch >= '0' && ch <= '9')
+            face_index = face_index * 10 + (ch - '0');
+        if(face_index >= face_num)
+            face_index = 0;
+        /* re-open the TTC file */
+        FT_New_Face(library, path, face_index, &face);
+    }
 
-    if (fterror != TT_Err_Ok) {
-      fprintf (stderr, "TT_Get_Face_Properties failed with %ld for file %s\n", fterror, path);
-      return fterror;
+    os2 = FT_Get_Sfnt_Table(face, ft_sfnt_os2);
+    if ( !os2 ) {
+        fprintf (stderr, "FT_Get_Sfnt_Table failed to load the OS/2 table for file %s\n", path);
+        return TTMKFS_USAGE;
+    }
+    post = FT_Get_Sfnt_Table(face, ft_sfnt_post);
+    if ( !post ) {
+        fprintf (stderr, "FT_Get_Sfnt_Table failed to load the PostScript table for file %s\n", path);
+        return TTMKFS_USAGE;
     }
 
     font_name    = get_name (face);
-    foundry_name = get_foundry (properties.os2->achVendID);
-    weight_name  = get_weight (path, &properties);
-    width_name   = get_width_name (path, &properties);
+    font_psname  = get_psname (face);
+    foundry_name = get_foundry (os2->achVendID);
+    weight_name  = get_weight (path, os2);
+    width_name   = get_width_name (path, os2);
 
     /* basic fone information */
     fprintf(outstream,"TTFINFO_FONT_FILE=\"%s\"\n",path);
+    fprintf(outstream,"TTFINFO_FACE_NUM=\"%lu\"\n",face_num);
+    fprintf(outstream,"TTFINFO_FACE_INDEX=\"%lu\"\n",face_index);
     fprintf(outstream,"TTFINFO_FONT_NAME=\"%s\"\n",font_name);
+    fprintf(outstream,"TTFINFO_FONT_PSNAME=\"%s\"\n",font_psname);
     fprintf(outstream,"TTFINFO_FOUNDRY_NAME=\"%s\"\n",foundry_name);
     fprintf(outstream,"TTFINFO_WEIGHT_NAME=\"%s\"\n",weight_name);
     fprintf(outstream,"TTFINFO_WIDTH=\"%s\"\n",width_name);
     
-    fprintf(outstream,"TTFINFO_NUMCMAP=\"%d\"\n",properties.num_CharMaps);
+    fprintf(outstream,"TTFINFO_NUMCMAP=\"%d\"\n",face->num_charmaps);
 
-    for (i = 0; i < properties.num_CharMaps; i++) {
-      TT_UShort platform;
-      TT_UShort encoding;
+    for (i = 0; i < face->num_charmaps; i++) {
+      FT_UShort platform;
+      FT_UShort encoding;
       
-      TT_Get_CharMap_ID (face, i, &platform, &encoding);
+      platform = face->charmaps[i]->platform_id;
+      encoding = face->charmaps[i]->encoding_id;
       
       /* get the cmap name */
       switch ( platform ) {
@@ -848,19 +890,17 @@ main (int argc, char *argv[])
       }
 
       /* print the cmap info... */
-      fprintf(outstream,"TTFINFO_CMAP%d=\"%d,%d\"\n",i,platform,encoding);
+      fprintf(outstream,"TTFINFO_CMAP%d=\"%d,%d\"\n", i, platform, encoding);
       /* print the cmap name */
-      fprintf(outstream,"TTFINFO_CMAPNAME%d=\"%s,%s\"\n",i,platStr, encoStr );
+      fprintf(outstream,"TTFINFO_CMAPNAME%d=\"%s,%s\"\n", i, platStr, encoStr );
 
-    /* fine the font string from the MS Map */
+    /* find the font string from the MS Map */
 
       if (platform == TT_PLATFORM_MICROSOFT) {
 	switch (encoding) {
 	case TT_MS_ID_UNICODE_CS:
 	  {
-
 	    /* check the 8-bit encoding */
-	    TT_CharMap cmap;
 	    struct {
 	      int *table;
 	      char *encoding;
@@ -887,14 +927,14 @@ main (int argc, char *argv[])
 	      {NULL,       NULL,         IGNORE}
 	    };
 	    
-	    TT_Get_CharMap (face, i, &cmap);
+	    fterror = FT_Set_Charmap(face, face->charmaps[i]);
 	    
 	    for (p = char_mappings; p->table; p++) {
 	      if ((p->completeness == IGNORE) || (p->completeness == completeness)) {
-		if (has_8bit_encoding (&cmap, p->table)) {
+		if (has_8bit_encoding (face, p->table)) {
 		  add_mapping (&head, path, foundry_name,
 			       font_name, weight_name, width_name,
-			       &properties, p->encoding); n++;
+			       os2, post, p->encoding); n++;
 		}
 	      }
 	    }
@@ -903,50 +943,50 @@ main (int argc, char *argv[])
 	       supported
 	       {
 	       int i;
-	       TT_ULong mask=1L;
+	       FT_ULong mask=1L;
 	       printf("%s\n",path);
 	       for(i=0;i<32;i++){
-	       if((properties.os2->ulUnicodeRange1) & mask)
+	       if((os2->ulUnicodeRange1) & mask)
 	       printf("bit %2d =1 !!\n",i);
 	       mask = mask << 1;
 	       }
 	       mask=1L;
 	       for(i=32;i<64;i++){
-	       if((properties.os2->ulUnicodeRange2) & mask)
+	       if((os2->ulUnicodeRange2) & mask)
 	       printf("bit %2d =1 !!\n",i);
 	       mask = mask << 1;
 	       }
 	       }
 	    */
 	    /* see the CJK encoding in the OS/2 table */
-	    if ( properties.os2->ulCodePageRange1 & TT_CODEPAGE_RANGE_932){
+	    if ( os2->ulCodePageRange1 & TT_CODEPAGE_RANGE_932){
 	      add_mapping (&head, path, foundry_name,
 			   font_name, weight_name, width_name,
-			   &properties, "jisx0208.1983-0");
+			   os2, post, "jisx0208.1983-0");
 	      n++;
 	    }
-	    if ( properties.os2->ulCodePageRange1 & TT_CODEPAGE_RANGE_936){
+	    if ( os2->ulCodePageRange1 & TT_CODEPAGE_RANGE_936){
 	      add_mapping (&head, path, foundry_name,
 			   font_name, weight_name, width_name,
-			   &properties, "gb2312.1980-0");
+			   os2, post, "gb2312.1980-0");
 	      n++;
 	    }
-	    if ( properties.os2->ulCodePageRange1 & TT_CODEPAGE_RANGE_950){
+	    if ( os2->ulCodePageRange1 & TT_CODEPAGE_RANGE_950){
 	      add_mapping (&head, path, foundry_name,
 			   font_name, weight_name, width_name,
-			   &properties, "big5-0");
+			   os2, post, "big5-0");
 	      n++;
 	    }
-	    if ( properties.os2->ulCodePageRange1 & TT_CODEPAGE_RANGE_949){
+	    if ( os2->ulCodePageRange1 & TT_CODEPAGE_RANGE_949){
 	      add_mapping (&head, path, foundry_name,
 			   font_name, weight_name, width_name,
-			   &properties, "ksc5601.1987-0");
+			   os2, post, "ksc5601.1987-0");
 	      n++;
 	    }
-	    if ( properties.os2->ulCodePageRange1 & TT_CODEPAGE_RANGE_1361){
+	    if ( os2->ulCodePageRange1 & TT_CODEPAGE_RANGE_1361){
 	      add_mapping (&head, path, foundry_name,
 			   font_name, weight_name, width_name,
-			   &properties, "ksc5601.1987-0");
+			   os2, post, "ksc5601.1987-0");
 	      n++;
 	    }
 	  }
@@ -954,39 +994,39 @@ main (int argc, char *argv[])
 	case TT_MS_ID_SJIS:
 	  add_mapping (&head, path, foundry_name,
 		       font_name, weight_name, width_name,
-		       &properties, "jisx0208.1983-0"); n++;
+		       os2, post, "jisx0208.1983-0"); n++;
 	  add_mapping (&head, path, foundry_name,
 		       font_name, weight_name, width_name,
-		       &properties, "jisx0201.1976-0"); n++;
+		       os2, post, "jisx0201.1976-0"); n++;
 	  break;
 	case TT_MS_ID_GB2312:
 	  add_mapping (&head, path, foundry_name,
 		       font_name, weight_name, width_name,
-		       &properties, "gb2312.1980-0"); n++;
+		       os2, post, "gb2312.1980-0"); n++;
 	  
 	  break;
 	case TT_MS_ID_BIG_5:
 	  add_mapping (&head, path, foundry_name,
 		       font_name, weight_name, width_name,
-		       &properties, "big5-0"); n++;
+		       os2, post, "big5-0"); n++;
 
 	  break;
 	case TT_MS_ID_WANSUNG:
 	  add_mapping (&head, path, foundry_name,
 		       font_name, weight_name, width_name,
-		       &properties, "ksc5601.1987-0"); n++;
+		       os2, post, "ksc5601.1987-0"); n++;
 	  
 	  break;
 	case TT_MS_ID_JOHAB:
 	  add_mapping (&head, path, foundry_name,
 		       font_name, weight_name, width_name,
-		       &properties, "ksc5601.1987-0"); n++;
+		       os2, post, "ksc5601.1987-0"); n++;
 	  
 	  break;
 	case TT_MS_ID_SYMBOL_CS:
 	  add_mapping (&head, path, foundry_name,
 		       font_name, weight_name, width_name,
-		       &properties, "microsoft-symbol"); n++;
+		       os2, post, "microsoft-symbol"); n++;
 	  break;
 	}
       }
@@ -995,11 +1035,11 @@ main (int argc, char *argv[])
     if (create_debug_entries) {
       add_mapping (&head, path, foundry_name,
 		   font_name, weight_name, width_name,
-		   &properties, "truetype-raw"); n++;
+		   os2, post, "truetype-raw"); n++;
     }
     
     /* we are done with this file */
-    TT_Close_Face (face);
+    FT_Done_Face (face);
     
     /* now write the number of mappings we actually got */
     fprintf(outstream,"TTFINFO_MAPNUM=\"%d\"\n", n);
@@ -1010,7 +1050,7 @@ main (int argc, char *argv[])
     }
 
     /* shut down */
-    TT_Done_FreeType (engine);
+    FT_Done_FreeType (library);
     fclose (outstream);
     
     return 0;
@@ -1083,7 +1123,7 @@ usage (const char *name)
 }
 
 const char *
-get_foundry (const TT_Char orig_code[4])
+get_foundry (const FT_Char orig_code[4])
 {
 
     char code[5];
@@ -1121,6 +1161,9 @@ get_foundry (const TT_Char orig_code[4])
 	{ "GLYF", "Glyph Systems"},
 	{ "GPI",  "Gamma Productions, Inc."},
 	{ "GPI ", "Gamma Productions, Inc."},
+	{ "HANY", "Hanyi"},     /* Hanyi, http://www.hanyi.com.cn/  */
+	{ "HGEC", "Huaguang"},  /* Beida Huaguang, http://www.hg.com.cn/ */
+	{ "HWAN", "Hwan"},      /* Hwan Design Inc. */
 	{ "HP",   "hp"},
 	{ "HP  ", "hp"},
 	{ "HY",   "HanYang System"},
@@ -1134,6 +1177,7 @@ get_foundry (const TT_Char orig_code[4])
 	{ "ITC",  "itc"},
 	{ "ITC ", "itc"},
 	{ "IMPR", "impress"},
+	{ "JBHG", "JBHG"},
 	{ "KATF", "katf"},
 	{ "LANS", "Lanston Type Co., Ltd."},
 	{ "LEAF", "interleaf"},
@@ -1187,17 +1231,14 @@ get_foundry (const TT_Char orig_code[4])
  */
 
 const char *
-get_name (TT_Face face)
+get_name (FT_Face face)
 {
     int i, j, k, n;
 
-    TT_UShort  platform;
-    TT_UShort  encoding;
-    TT_UShort  language;
-    TT_UShort  NameId;
+    FT_SfntName        aname;
 
     /* get the number of name strings in this font */
-    if ((n = TT_Get_Name_Count (face)) < 0) {
+    if ((n = FT_Get_Sfnt_Name_Count (face)) < 0) {
 	return NULL;
     }
     /* printf("\nn=%d\n",n); */
@@ -1205,70 +1246,66 @@ get_name (TT_Face face)
        entries; and at the second time at TT_NAME_ID_PS_NAME. Of course the
        function returns as soon as an english name for the font is found,
        so most of the time one loop is enough; however some fonts have only
-       non-english names in their font faminly entry, so the second loop
-       to try the Postscript name as last ressort
+       non-english names in their font family entry, so the second loop
+       to try the Postscript name as last resort
     */
     for (k = 0; k < 2; k++) {
 
 	/* now look at every entry */
 	for (i = 0; i < n; i++) {
-	    TT_Get_Name_ID (face, i, &platform, &encoding, &language, &NameId);
-	    /*	    printf("k=%d, face=%d, i=%d, platform=%d, encoding=%d, lang=%d, Id=%d\n",k ,face, i, platform, encoding, language, NameId); */
-	    if (((k == 0) && (NameId == TT_NAME_ID_FONT_FAMILY)) ||
-		((k == 1) && (NameId == TT_NAME_ID_PS_NAME))) {
+	    FT_Get_Sfnt_Name (face, i, &aname);
+	    /*	    printf("k=%d, face=%d, i=%d, platform=%d, encoding=%d, lang=%d, Id=%d\n",k ,face, i, platform, encoding, language, aname.name_id); */
+	    if (((k == 0) && (aname.name_id == TT_NAME_ID_FONT_FAMILY)) ||
+		((k == 1) && (aname.name_id == TT_NAME_ID_PS_NAME))) {
 
 		/* Try a Macintosh english name */
-		if ((platform == TT_PLATFORM_MACINTOSH) && 
-		    (language == TT_MAC_LANGID_ENGLISH))
+		if ((aname.platform_id == TT_PLATFORM_MACINTOSH) && 
+		    (aname.language_id == TT_MAC_LANGID_ENGLISH))
 		    {
-			char*  string;
-			TT_UShort string_len;
+			FT_UInt string_len;
 			static char  name_buffer[257];
 			int   name_len;
 			int   english=1;
 
-			TT_Get_Name_String (face, i, &string, &string_len);
-			string_len =  string_len < 257 ? string_len : 257;
+			string_len = aname.string_len < 257 ? aname.string_len : 257;
 			name_len = 0;
 
 			for (j = 0; j < string_len; j++) {
 			    /* some font names claim to be in english but aren't */
-			    if ((string[j] & 0xff) > 0x7f) english=0;
+			    if ((aname.string[j] & 0xff) > 0x7f) english=0;
 			    /* '-' is reserved in XLFD */
 			    name_buffer[name_len++] =
-				(string[j] == '-') ? '_' : string[j];
+				(aname.string[j] == '-') ? '_' : aname.string[j];
 			}
 
 			name_buffer[name_len] = '\0';
 			if (english) return name_buffer;
 		    }
 		/* Try to find an Apple Unicode or Microsoft English name */
-		if ((platform == TT_PLATFORM_APPLE_UNICODE) ||
-		    ((platform == TT_PLATFORM_MICROSOFT) &&
-		     ((language & 0x00FF) ==
+		if ((aname.platform_id == TT_PLATFORM_APPLE_UNICODE) ||
+		    ((aname.platform_id == TT_PLATFORM_MICROSOFT) &&
+		     ((aname.language_id & 0x00FF) ==
 		      (TT_MS_LANGID_ENGLISH_UNITED_STATES & 0x00FF))) ||
-		    (((platform == TT_PLATFORM_MICROSOFT) &&
-		      (encoding == TT_MS_ID_UNICODE_CS)) &&
-		     (NameId == TT_NAME_ID_PS_NAME)))
+		    (((aname.platform_id == TT_PLATFORM_MICROSOFT) &&
+		      (aname.encoding_id == TT_MS_ID_UNICODE_CS)) &&
+		     (aname.name_id == TT_NAME_ID_PS_NAME)))
 		    {
 			/* The following code was inspired from Mark Leisher's */
 			/* ttf2bdf package                                     */
-			char*  string;
-			TT_UShort string_len;
+			FT_UInt string_len;
 			static char  name_buffer[513];
 			int   name_len;
 
-			TT_Get_Name_String (face, i, &string, &string_len);
-			string_len =  string_len < 512 ? string_len : 512;
+			string_len = aname.string_len < 512 ? aname.string_len : 512;
 			name_len = 0;
 			/* convert from unicode to ASCII */
 			for (j = 0; j < string_len; j ++) {
 			  /* drop the NULL part, dont't use j+=2 to avoid
 			     endian problem. */
-			  if (string[j] != '\0'){
+			  if (aname.string[j] != '\0'){
 			    /* '-' is reserved in XLFD */
 			    name_buffer[name_len++] =
-			      (string[j] == '-') ? '_' : string[j];
+			      (aname.string[j] == '-') ? '_' : aname.string[j];
 			  }
 			}
 			name_buffer[name_len] = '\0';
@@ -1282,19 +1319,106 @@ get_name (TT_Face face)
     return "unknown";
 }
 
+/* This get_psname() function is copied from get_name().  I probably should
+   try to merge the two functions somehow, but unfortunately, I do not have
+   the time right now.  :-)  Anthony Fok <anthony@thizlinux.com>
+*/
+
+const char *
+get_psname (FT_Face face)
+{
+    int i, j, k, n;
+
+    FT_SfntName  aname;
+
+    /* get the number of name strings in this font */
+    if ((n = FT_Get_Sfnt_Name_Count (face)) < 0) {
+	return NULL;
+    }
+    /* printf("\nn=%d\n",n); */
+
+    /* for (k = 0; k < 2; k++) { */
+
+    k = 1;
+	/* now look at every entry */
+	for (i = 0; i < n; i++) {
+	    FT_Get_Sfnt_Name (face, i, &aname);
+	    /*	    printf("k=%d, face=%d, i=%d, platform=%d, encoding=%d, lang=%d, Id=%d\n",k ,face, i, platform, encoding, language, aname.name_id); */
+
+	    if (aname.name_id == TT_NAME_ID_PS_NAME) {
+
+		/* Try a Macintosh english name */
+		if ((aname.platform_id == TT_PLATFORM_MACINTOSH) && 
+		    (aname.language_id == TT_MAC_LANGID_ENGLISH))
+		    {
+			FT_UInt string_len;
+			static char  name_buffer[257];
+			int   name_len;
+			int   english=1;
+
+			string_len = aname.string_len < 257 ? aname.string_len : 257;
+			name_len = 0;
+
+			for (j = 0; j < string_len; j++) {
+			    /* some font names claim to be in english but aren't */
+			    if ((aname.string[j] & 0xff) > 0x7f) english=0;
+			    name_buffer[name_len++] = aname.string[j];
+			}
+
+			name_buffer[name_len] = '\0';
+			if (english) return name_buffer;
+		    }
+		/* Try to find an Apple Unicode or Microsoft English name */
+		if ((aname.platform_id == TT_PLATFORM_APPLE_UNICODE) ||
+		    ((aname.platform_id == TT_PLATFORM_MICROSOFT) &&
+		     ((aname.language_id & 0x00FF) ==
+		      (TT_MS_LANGID_ENGLISH_UNITED_STATES & 0x00FF))) ||
+		    (((aname.platform_id == TT_PLATFORM_MICROSOFT) &&
+		      (aname.encoding_id == TT_MS_ID_UNICODE_CS)) &&
+		     (aname.name_id == TT_NAME_ID_PS_NAME)))
+		    {
+			/* The following code was inspired from Mark Leisher's */
+			/* ttf2bdf package                                     */
+			FT_UInt string_len;
+			static char  name_buffer[513];
+			int   name_len;
+			int   english=1;
+
+			string_len = aname.string_len < 512 ? aname.string_len : 512;
+			name_len = 0;
+			/* convert from unicode to ASCII */
+			for (j = 0; j < string_len; j ++) {
+			  /* drop the NULL part, dont't use j+=2 to avoid
+			     endian problem. */
+			  if (aname.string[j] != '\0'){
+			    if ((aname.string[j] & 0xff) > 0x7f) english=0;
+			    name_buffer[name_len++] = aname.string[j];
+			  }
+			}
+			name_buffer[name_len] = '\0';
+			if (english) return name_buffer;
+		    }
+	    }
+    }
+ 
+    /* Not found */
+    return "unknown";
+}
+
 static void  add_mapping (struct xlfdmapping **listhead,
 			  char *file_name,
 			  const char *foundry_name,
 			  const char *font_name,
 			  const char *weight_name,
 			  const char *width_name,
-			  TT_Face_Properties *properties,
+			  TT_OS2 *os2,
+			  TT_Postscript *post,
 			  char *encodingscheme
 			  )
 {
     struct xlfdmapping *p;
     char char_for_spacing;
-    
+ 
     /* allocate space for the file to xlfd mapping */
     p = malloc (sizeof (struct xlfdmapping));
 
@@ -1314,29 +1438,41 @@ static void  add_mapping (struct xlfdmapping **listhead,
      * count and collect all our mappings.
      */
 
-    if ((strncmp("jisx",encodingscheme,4)==0) ||
+    /*if ((strncmp("jisx",encodingscheme,4)==0) ||
 	(strncmp("gb2312",encodingscheme,5)==0) ||
 	(strncmp("big5",encodingscheme,4)==0) ||
 	(strncmp("ksc",encodingscheme,3)==0))
 	{
 	    char_for_spacing = 'c';
 	} else {
-	    char_for_spacing = (properties->postscript->isFixedPitch) ?
+	    char_for_spacing = (post->isFixedPitch) ?
 		'm' : 'p' ;
-	}
+	}*/
+    char_for_spacing = (post->isFixedPitch) ? 'm' : 'p';
 
     sprintf (p->mapping, "-%s-%s-%s-%c-%s--0-0-0-0-%c-0-%s",
 	     foundry_name,
 	     font_name,
 	     weight_name,
-	     properties->os2->fsSelection & TT_SELECTION_ITALIC ? 'i' : 'r',
+	     os2->fsSelection & TT_SELECTION_ITALIC ? 'i' : 'r',
 	     width_name,
 	     char_for_spacing,
 	     encodingscheme);
 }
 
+static void  free_mappings (struct xlfdmapping **listhead)
+{
+    struct xlfdmapping *p;
+    p = *listhead;
+    while (p) {
+      p = p->next;
+      free(*listhead);
+      *listhead = p;
+    }
+}
+
 int
-has_8bit_encoding (TT_CharMap *cmap, const int *table)
+has_8bit_encoding (FT_Face face, const int *table)
 {
     int i;
     int missing = 0;
@@ -1345,7 +1481,7 @@ has_8bit_encoding (TT_CharMap *cmap, const int *table)
 	if (table[i] < 0) 
 	    continue;
 
-	if (TT_Char_Index (*cmap, table[i]) == 0) {
+	if (FT_Get_Char_Index (face, table[i]) == 0) {
 	    missing++;
 	}
     }
@@ -1354,7 +1490,7 @@ has_8bit_encoding (TT_CharMap *cmap, const int *table)
 }
 
 static const char *
-panose_get_weight (TT_Byte *panose)
+panose_get_weight (FT_Byte *panose)
 {
     static char *panose_weight_latin[] = {
 	"any",
@@ -1386,7 +1522,7 @@ panose_get_weight (TT_Byte *panose)
 }
 
 static const char *
-os2_get_weight (TT_UShort usWeightClass)
+os2_get_weight (FT_UShort usWeightClass)
 {
     switch (usWeightClass) {
     case TT_FW_THIN: return "thin";
@@ -1404,26 +1540,26 @@ os2_get_weight (TT_UShort usWeightClass)
 }
 
 static const char *
-get_weight (const char *name, TT_Face_Properties *properties)
+get_weight (const char *name, TT_OS2 *os2)
 {
     const char *result;
     
-    if (use_panose && ((result = panose_get_weight (properties->os2->panose)) != NULL)) {
+    if (use_panose && ((result = panose_get_weight (os2->panose)) != NULL)) {
 	return result;
     } else {
-	if ((result = os2_get_weight (properties->os2->usWeightClass)) != NULL) {
+	if ((result = os2_get_weight (os2->usWeightClass)) != NULL) {
 	    return result;
 	}
 	
-	fprintf (stderr, "font %s: unknown usWeightClass %hu\n", name, properties->os2->usWeightClass);
+	fprintf (stderr, "font %s: unknown usWeightClass %hu\n", name, os2->usWeightClass);
     }
 
     /* last fall back */
-    return properties->os2->fsSelection & (1 << 5) ? "bold" : "medium";
+    return os2->fsSelection & (1 << 5) ? "bold" : "medium";
 }
 
 static const char *
-panose_get_width_name (TT_Byte *panose)
+panose_get_width_name (FT_Byte *panose)
 {
     static char *panose_spacing_latin_text [] = {
 	"any",
@@ -1471,7 +1607,7 @@ panose_get_width_name (TT_Byte *panose)
 }
 
 static const char *
-os2_get_width_name (TT_UShort usWidthClass)
+os2_get_width_name (FT_UShort usWidthClass)
 {
     switch (usWidthClass) {
     case TT_FWIDTH_ULTRA_CONDENSED: return "ultra condensed";
@@ -1489,18 +1625,18 @@ os2_get_width_name (TT_UShort usWidthClass)
 }
 
 static const char *
-get_width_name (const char *name, TT_Face_Properties *properties)
+get_width_name (const char *name, TT_OS2 *os2)
 {
     const char *result;
     
-    if (use_panose && ((result = panose_get_width_name (properties->os2->panose)) != NULL)) {
+    if (use_panose && ((result = panose_get_width_name (os2->panose)) != NULL)) {
 	return result;
     } else {
-	if ((result = os2_get_width_name (properties->os2->usWidthClass)) != NULL) {
+	if ((result = os2_get_width_name (os2->usWidthClass)) != NULL) {
 	    return result;
 	}
 	
-	fprintf (stderr, "font %s: unknown usWidthClass %hu\n", name, properties->os2->usWidthClass);
+	fprintf (stderr, "font %s: unknown usWidthClass %hu\n", name, os2->usWidthClass);
     }
 
     /* last fall back */
